package org.expeditee.gio.swing;

public class SwingConversions {
	
	public static org.expeditee.core.Dimension fromSwingDimension(java.awt.Dimension dimension)
	{
		if (dimension == null) return null;
		
		return new org.expeditee.core.Dimension(dimension.width, dimension.height);
	}
	
	public static java.awt.Dimension toSwingDimension(org.expeditee.core.Dimension dimension)
	{
		if (dimension == null) return null;
		
		return new java.awt.Dimension(dimension.width, dimension.height);
	}
	
	public static java.awt.Color toSwingColor(org.expeditee.core.Colour c)
	{
		if (c == null) return null;
		
		int r = c.getRed255();
		int g = c.getGreen255();
		int b = c.getBlue255();
		int a = c.getAlpha255();
		
		return new java.awt.Color(r, g, b, a);
	}
	
	public static org.expeditee.core.Colour fromSwingColor(java.awt.Color c)
	{
		if (c == null) return null;
		
		return org.expeditee.core.Colour.FromRGBA255(c.getRed(), c.getGreen(), c.getBlue(), c.getAlpha());
	}
	
	public static java.awt.Point toSwingPoint(org.expeditee.core.Point point)
	{
		if (point == null) return null;
		
		return new java.awt.Point(point.getX(), point.getY());
	}
	
	public static org.expeditee.core.Point fromSwingPoint(java.awt.Point point)
	{
		if (point == null) return null;
		
		return new org.expeditee.core.Point(point.x, point.y);
	}
	
	public static org.expeditee.core.Point fromSwingPoint(java.awt.geom.Point2D point)
	{
		if (point == null) return null;
		
		return new org.expeditee.core.Point((int) point.getX(), (int) point.getY());
	}
	
	public static java.awt.Rectangle toSwingRectangle(org.expeditee.core.bounds.AxisAlignedBoxBounds b)
	{
		if (b == null) return null;
		
		return new java.awt.Rectangle(b.getMinX(), b.getMinY(), b.getWidth(), b.getHeight());
	}
	
	public static org.expeditee.core.bounds.AxisAlignedBoxBounds fromSwingRectangle(java.awt.Rectangle r)
	{
		if (r == null) return null;
		
		return new org.expeditee.core.bounds.AxisAlignedBoxBounds(r.x, r.y, r.width, r.height);
	}
	
	public static java.awt.font.TextHitInfo toSwingTextHitInfo(org.expeditee.core.TextHitInfo thi)
	{
		if (thi == null) return null;
		
		if (thi.isLeadingEdge) {
			return java.awt.font.TextHitInfo.leading(thi.charIndex);
		} else {
			return java.awt.font.TextHitInfo.trailing(thi.charIndex);
		}
	}

	public static org.expeditee.core.TextHitInfo fromSwingTextHitInfo(java.awt.font.TextHitInfo swingThi)
	{
		if (swingThi == null) return null;
		
		return new org.expeditee.core.TextHitInfo(swingThi.getCharIndex(), swingThi.isLeadingEdge());
	}
	
	public static int toSwingFontStyle(org.expeditee.core.Font.Style style)
	{
		switch (style) {
		case PLAIN:
			return java.awt.Font.PLAIN;
		case BOLD:
			return java.awt.Font.BOLD;
		case ITALIC:
			return java.awt.Font.ITALIC;
		case BOLD_ITALIC:
			return java.awt.Font.BOLD | java.awt.Font.ITALIC;
		default:
			return -1;	
		}
	}
	
	public static org.expeditee.core.Font.Style fromSwingFontStyle(int style)
	{
		switch (style) {
		case java.awt.Font.BOLD | java.awt.Font.ITALIC:
			return org.expeditee.core.Font.Style.BOLD_ITALIC;
		case java.awt.Font.BOLD:
			return org.expeditee.core.Font.Style.BOLD;
		case java.awt.Font.ITALIC:
			return org.expeditee.core.Font.Style.ITALIC;
		case java.awt.Font.PLAIN:
			return org.expeditee.core.Font.Style.PLAIN;
		default:
			return null;
		}
	}
	
	public static int toSwingCursorType(org.expeditee.core.Cursor.CursorType type)
	{
		switch (type) {
			case DEFAULT:
				return java.awt.Cursor.DEFAULT_CURSOR;
			case TEXT:
				return java.awt.Cursor.TEXT_CURSOR;
			case CROSSHAIR:
				return java.awt.Cursor.CROSSHAIR_CURSOR;
			default:
				return java.awt.Cursor.CUSTOM_CURSOR;
		}
	}
	
	public static org.expeditee.core.Cursor.CursorType fromSwingCursorType(int type)
	{
		switch (type) {
		case java.awt.Cursor.DEFAULT_CURSOR:
			return org.expeditee.core.Cursor.CursorType.DEFAULT;
		case java.awt.Cursor.TEXT_CURSOR:
			return org.expeditee.core.Cursor.CursorType.TEXT;
		case java.awt.Cursor.CROSSHAIR_CURSOR:
			return org.expeditee.core.Cursor.CursorType.CROSSHAIR;
		case java.awt.Cursor.CUSTOM_CURSOR:
			return org.expeditee.core.Cursor.CursorType.CUSTOM;
		default:
			return null;
		}
	}
	
	public static org.expeditee.core.GradientFill fromSwingGradientPaint(java.awt.GradientPaint paint)
	{
		if (paint == null) return null;
		
		org.expeditee.core.Colour colour1 = fromSwingColor(paint.getColor1());
		org.expeditee.core.Point point1 = fromSwingPoint(paint.getPoint1());
		org.expeditee.core.Colour colour2 = fromSwingColor(paint.getColor2());
		org.expeditee.core.Point point2 = fromSwingPoint(paint.getPoint2());
		
		return new org.expeditee.core.GradientFill(colour1, point1, colour2, point2, paint.isCyclic());
	}
	
	public static java.awt.GradientPaint toSwingGradientPaint(org.expeditee.core.GradientFill fill)
	{
		if (fill == null) return null;
		
		java.awt.Color color1 = toSwingColor(fill.getFromColour());
		java.awt.Color color2 = toSwingColor(fill.getToColour());
		
		return new java.awt.GradientPaint(fill.getFromPoint().getX(), fill.getFromPoint().getY(), color1, fill.getToPoint().getX(), fill.getToPoint().getY(), color2, !fill.isAcyclic());
	}
	
	public static org.expeditee.core.Stroke fromSwingStroke(java.awt.BasicStroke basicStroke)
	{
		if (basicStroke == null) return null;
		
		org.expeditee.core.Stroke stroke = new org.expeditee.core.Stroke();
		stroke.thickness = basicStroke.getLineWidth();
		stroke.capType = fromSwingCapType(basicStroke.getEndCap());
		stroke.joinType = fromSwingJoinType(basicStroke.getLineJoin());
		stroke.dashArray = basicStroke.getDashArray().clone();
		stroke.dashPhase = basicStroke.getDashPhase();
		
		return stroke;
	}
	
	public static java.awt.BasicStroke toSwingStroke(org.expeditee.core.Stroke stroke)
	{
		if (stroke == null) return null;
		
		java.awt.BasicStroke basicStroke = new java.awt.BasicStroke(stroke.thickness,
																	toSwingCapType(stroke.capType),
																	toSwingJoinType(stroke.joinType),
																	10.0f, // This is the default for miterLimit
																	stroke.dashArray.clone(),
																	stroke.dashPhase);
		
		return basicStroke;
	}
	
	public static org.expeditee.core.Stroke.CAP fromSwingCapType(int swingCapType)
	{
		switch(swingCapType) {
			case java.awt.BasicStroke.CAP_BUTT:
				return org.expeditee.core.Stroke.CAP.BUTT;
			case java.awt.BasicStroke.CAP_ROUND:
				return org.expeditee.core.Stroke.CAP.ROUND;
			case java.awt.BasicStroke.CAP_SQUARE:
				return org.expeditee.core.Stroke.CAP.SQUARE;
			default:
				return null;
		}
	}
	
	public static int toSwingCapType(org.expeditee.core.Stroke.CAP cap)
	{
		switch(cap) {
			case BUTT:
				return java.awt.BasicStroke.CAP_BUTT;
			case ROUND:
				return java.awt.BasicStroke.CAP_ROUND;
			case SQUARE:
				return java.awt.BasicStroke.CAP_SQUARE;
			default:
				return -1;
		}
	}
	
	public static org.expeditee.core.Stroke.JOIN fromSwingJoinType(int swingJoinType)
	{
		switch(swingJoinType) {
			case java.awt.BasicStroke.JOIN_BEVEL:
				return org.expeditee.core.Stroke.JOIN.BEVEL;
			case java.awt.BasicStroke.JOIN_MITER:
				return org.expeditee.core.Stroke.JOIN.MITER;
			case java.awt.BasicStroke.JOIN_ROUND:
				return org.expeditee.core.Stroke.JOIN.ROUND;
			default:
				return null;
		}
	}
	
	public static int toSwingJoinType(org.expeditee.core.Stroke.JOIN join)
	{
		switch(join) {
			case BEVEL:
				return java.awt.BasicStroke.JOIN_BEVEL;
			case MITER:
				return java.awt.BasicStroke.JOIN_MITER;
			case ROUND:
				return java.awt.BasicStroke.JOIN_ROUND;
			default:
				return -1;
		}
	}
	
	public static org.expeditee.core.bounds.PolygonBounds fromSwingPolygon(java.awt.Polygon polygon)
	{
		org.expeditee.core.bounds.PolygonBounds polygonBounds = new org.expeditee.core.bounds.PolygonBounds();
		
		for (int i = 0; i < polygon.npoints; i++) {
			polygonBounds.addPoint(polygon.xpoints[i], polygon.ypoints[i]);
		}
		
		return polygonBounds;
	}
	
	public static java.awt.Polygon toSwingPolygon(org.expeditee.core.bounds.PolygonBounds polygonBounds)
	{
		java.awt.Polygon polygon = new java.awt.Polygon();
		
		for (int i = 0; i < polygonBounds.getPointCount(); i++) {
			org.expeditee.core.Point p = polygonBounds.getPoint(i);
			polygon.addPoint(p.getX(), p.getY());
		}
		
		return polygon;
	}
	
	public static int toSwingMouseButton(org.expeditee.gio.input.KBMInputEvent.MouseButton mouseButton)
	{
		switch (mouseButton) {
		case LEFT:
			return java.awt.event.MouseEvent.BUTTON1;
		case MIDDLE:
			return java.awt.event.MouseEvent.BUTTON2;
		case RIGHT:
			return java.awt.event.MouseEvent.BUTTON3;
		default:
			return java.awt.event.MouseEvent.NOBUTTON;
		}
	}
	
	public static org.expeditee.gio.input.KBMInputEvent.Key fromSwingVirtualKey(int swingKey)
	{
		switch (swingKey) {
		case java.awt.event.KeyEvent.VK_SHIFT: return org.expeditee.gio.input.KBMInputEvent.Key.SHIFT;
		case java.awt.event.KeyEvent.VK_CONTROL: return org.expeditee.gio.input.KBMInputEvent.Key.CTRL;
		case java.awt.event.KeyEvent.VK_ALT: return org.expeditee.gio.input.KBMInputEvent.Key.ALT;
		case java.awt.event.KeyEvent.VK_ESCAPE: return org.expeditee.gio.input.KBMInputEvent.Key.ESC;
		case java.awt.event.KeyEvent.VK_F1: return org.expeditee.gio.input.KBMInputEvent.Key.F1;
		case java.awt.event.KeyEvent.VK_F2: return org.expeditee.gio.input.KBMInputEvent.Key.F2;
		case java.awt.event.KeyEvent.VK_F3: return org.expeditee.gio.input.KBMInputEvent.Key.F3;
		case java.awt.event.KeyEvent.VK_F4: return org.expeditee.gio.input.KBMInputEvent.Key.F4;
		case java.awt.event.KeyEvent.VK_F5: return org.expeditee.gio.input.KBMInputEvent.Key.F5;
		case java.awt.event.KeyEvent.VK_F6: return org.expeditee.gio.input.KBMInputEvent.Key.F6;
		case java.awt.event.KeyEvent.VK_F7: return org.expeditee.gio.input.KBMInputEvent.Key.F7;
		case java.awt.event.KeyEvent.VK_F8: return org.expeditee.gio.input.KBMInputEvent.Key.F8;
		case java.awt.event.KeyEvent.VK_F9: return org.expeditee.gio.input.KBMInputEvent.Key.F9;
		case java.awt.event.KeyEvent.VK_F10: return org.expeditee.gio.input.KBMInputEvent.Key.F10;
		case java.awt.event.KeyEvent.VK_F11: return org.expeditee.gio.input.KBMInputEvent.Key.F11;
		case java.awt.event.KeyEvent.VK_F12: return org.expeditee.gio.input.KBMInputEvent.Key.F12;
		case java.awt.event.KeyEvent.VK_UP: return org.expeditee.gio.input.KBMInputEvent.Key.UP_ARROW;
		case java.awt.event.KeyEvent.VK_DOWN: return org.expeditee.gio.input.KBMInputEvent.Key.DOWN_ARROW;
		case java.awt.event.KeyEvent.VK_LEFT: return org.expeditee.gio.input.KBMInputEvent.Key.LEFT_ARROW;
		case java.awt.event.KeyEvent.VK_RIGHT: return org.expeditee.gio.input.KBMInputEvent.Key.RIGHT_ARROW;
		case java.awt.event.KeyEvent.VK_HOME: return org.expeditee.gio.input.KBMInputEvent.Key.HOME;
		case java.awt.event.KeyEvent.VK_END: return org.expeditee.gio.input.KBMInputEvent.Key.END;
		case java.awt.event.KeyEvent.VK_DELETE: return org.expeditee.gio.input.KBMInputEvent.Key.DELETE;
		case java.awt.event.KeyEvent.VK_PAGE_UP: return org.expeditee.gio.input.KBMInputEvent.Key.PAGE_UP;
		case java.awt.event.KeyEvent.VK_PAGE_DOWN: return org.expeditee.gio.input.KBMInputEvent.Key.PAGE_DOWN;
		case java.awt.event.KeyEvent.VK_BACK_SPACE: return org.expeditee.gio.input.KBMInputEvent.Key.BACKSPACE;
		case java.awt.event.KeyEvent.VK_TAB: return org.expeditee.gio.input.KBMInputEvent.Key.TAB;
		case java.awt.event.KeyEvent.VK_A: return org.expeditee.gio.input.KBMInputEvent.Key.A;
		case java.awt.event.KeyEvent.VK_B: return org.expeditee.gio.input.KBMInputEvent.Key.B;
		case java.awt.event.KeyEvent.VK_C: return org.expeditee.gio.input.KBMInputEvent.Key.C;
		case java.awt.event.KeyEvent.VK_D: return org.expeditee.gio.input.KBMInputEvent.Key.D;
		case java.awt.event.KeyEvent.VK_E: return org.expeditee.gio.input.KBMInputEvent.Key.E;
		case java.awt.event.KeyEvent.VK_F: return org.expeditee.gio.input.KBMInputEvent.Key.F;
		case java.awt.event.KeyEvent.VK_G: return org.expeditee.gio.input.KBMInputEvent.Key.G;
		case java.awt.event.KeyEvent.VK_H: return org.expeditee.gio.input.KBMInputEvent.Key.H;
		case java.awt.event.KeyEvent.VK_I: return org.expeditee.gio.input.KBMInputEvent.Key.I;
		case java.awt.event.KeyEvent.VK_J: return org.expeditee.gio.input.KBMInputEvent.Key.J;
		case java.awt.event.KeyEvent.VK_K: return org.expeditee.gio.input.KBMInputEvent.Key.K;
		case java.awt.event.KeyEvent.VK_L: return org.expeditee.gio.input.KBMInputEvent.Key.L;
		case java.awt.event.KeyEvent.VK_M: return org.expeditee.gio.input.KBMInputEvent.Key.M;
		case java.awt.event.KeyEvent.VK_N: return org.expeditee.gio.input.KBMInputEvent.Key.N;
		case java.awt.event.KeyEvent.VK_O: return org.expeditee.gio.input.KBMInputEvent.Key.O;
		case java.awt.event.KeyEvent.VK_P: return org.expeditee.gio.input.KBMInputEvent.Key.P;
		case java.awt.event.KeyEvent.VK_Q: return org.expeditee.gio.input.KBMInputEvent.Key.Q;
		case java.awt.event.KeyEvent.VK_R: return org.expeditee.gio.input.KBMInputEvent.Key.R;
		case java.awt.event.KeyEvent.VK_S: return org.expeditee.gio.input.KBMInputEvent.Key.S;
		case java.awt.event.KeyEvent.VK_T: return org.expeditee.gio.input.KBMInputEvent.Key.T;
		case java.awt.event.KeyEvent.VK_U: return org.expeditee.gio.input.KBMInputEvent.Key.U;
		case java.awt.event.KeyEvent.VK_V: return org.expeditee.gio.input.KBMInputEvent.Key.V;
		case java.awt.event.KeyEvent.VK_W: return org.expeditee.gio.input.KBMInputEvent.Key.W;
		case java.awt.event.KeyEvent.VK_X: return org.expeditee.gio.input.KBMInputEvent.Key.X;
		case java.awt.event.KeyEvent.VK_Y: return org.expeditee.gio.input.KBMInputEvent.Key.Y;
		case java.awt.event.KeyEvent.VK_Z: return org.expeditee.gio.input.KBMInputEvent.Key.Z;
		case java.awt.event.KeyEvent.VK_ENTER: return org.expeditee.gio.input.KBMInputEvent.Key.ENTER;
		default: return null;
		}
	}
}
